C Work performed under the auspices of the U.S. Department of Energy
C by Lawrence Livermore National Laboratory under contract number 
C W-7405-Eng-48.
C
C Copyright 1990 the Regents of the University of California.
C All rights reserved.
C
      SUBROUTINE DDINI(X,Y,YPRIME,NEQ,
     *   RES,JAC,PSOL,H,WT,IDID,RPAR,IPAR,
     *   PHI,SAVR,DELTA,E,WM,IWM,
     *   HMIN,UROUND,EPLI,SQRTN,RSQRTN,EPCON,
     *   JFLG,NONNEG,NTYPE,NTEMP,NLSI)
C
C***BEGIN PROLOGUE  DDINI
C***REFER TO  DDASPK
C***DATE WRITTEN   891219   (YYMMDD)
C***REVISION DATE  910624   (YYMMDD)
C
C
C-----------------------------------------------------------------------
C***DESCRIPTION
C
C     DDINI takes one step of size H or smaller
C     to find YPRIME.  X and Y are updated to be consistent
C     with the new step.
C     The routine NLSI solves the resulting nonlinear system(s).
C
C     The initial guess for YPRIME is not involved in the error test.
C     This may have trouble converging if the initial guess is no
C     good, or if G(X,Y,YPRIME) depends nonlinearly on YPRIME.
C
C     The parameters represent
C
C     X  --        Independent variable.
C     Y  --        Solution vector at X.
C     YPRIME --    Derivative of solution vector.
C     NEQ --       Number of equations to be integrated.
C     RES --       External user-supplied subroutine
C                  to evaluate the residual.  See RES description
C                  in DDASPK prologue.
C     JAC --       External user-supplied routine to update
C                  Jacobian or preconditioner information in the
C                  nonlinear solver.  See JAC description in DDASPK
C                  prologue.
C     PSOL --      External user-supplied routine to solve
C                  a linear system using preconditioning. 
C                  (This is optional).  See PSOL in DDASPK prologue.
C     H --         Stepsize. DDINI may reduce H if it needs to.
C     WT --        Vector of weights for error criterion.
C     IDID --      Completion code returned from the nonlinear solver.
C                  See IDID description in DDASPK prologue.
C     RPAR,IPAR -- Real and integer parameter arrays that
C                  are used for communication between the
C                  calling program and external user routines.
C                  They are not altered by DDINI.
C     PHI --       Work space for DDINI of length NEQ.
C     SAVR --      Work vector for DDINI of length NEQ.
C     DELTA,E --   Work vectors for DDINI of length NEQ.
C     WM,IWM --    Real and integer arrays storing
C                  information required by the linear solver.
C
C     The other parameters are information
C     which is needed internally by DDINI.
C
C-----------------------------------------------------------------------
C***ROUTINES CALLED
C   DDWNRM, NLSI
C
C***END PROLOGUE  DDINI
C
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Y(*),YPRIME(*),WT(*)
      DIMENSION PHI(NEQ,*),SAVR(*),DELTA(*),E(*)
      DIMENSION WM(*),IWM(*)
      DIMENSION RPAR(*),IPAR(*)
      EXTERNAL  RES, JAC, PSOL, NLSI
C
      PARAMETER (LETF=14)
      PARAMETER (LCFN=15)
C
C
C
C
C
C-----------------------------------------------------------------------
C     BLOCK 1.
C     Initializations.
C-----------------------------------------------------------------------
C
      IDID=1
      NEF=0
      NCF=0
      XOLD = X
C
C     SAVE Y AND YPRIME IN PHI
      DO 100 I=1,NEQ
         PHI(I,1)=Y(I)
100      PHI(I,2)=YPRIME(I)
C
C
C
C
C
C-----------------------------------------------------------------------
C     BLOCK 2
C     Compute necessary coefficients for
C     this step.
C-----------------------------------------------------------------------
C
200   CONTINUE
      CJ=1.0D0/H
C
C     Update time
C
      X = X+H
C
C     Reset IDID to 1
C
      IDID = 1
C
C
C
C
C
C-----------------------------------------------------------------------
C     BLOCK 3
C     Call the nonlinear system solver to obtain
C     consistent initial values for Y and YPRIME.
C-----------------------------------------------------------------------
C
      CALL NLSI(X,Y,YPRIME,NEQ,
     *   RES,JAC,PSOL,H,WT,JSDUM,IDID,RPAR,IPAR,PHIDUM,GADUM,
     *   SAVR,DELTA,E,WM,IWM,CJ,CJDUM,CJLDUM,SDUM,
     *   UROUND,EPLI,SQRTN,RSQRTN,EPCON,JCDUM,JFLG,KPDUM,
     *   NONNEG,NTEMP,NTYPE,IERNLS)
C
      IF(IERNLS .NE. 0)GO TO 600
C
C
C
C
C
C-----------------------------------------------------------------------
C     BLOCK 4
C     Estimate the local error and test
C     whether this step is successful.
C-----------------------------------------------------------------------
C
      DO 400 I=1,NEQ
400      E(I)=Y(I)-PHI(I,1)
      ERR=DDWNRM(NEQ,E,WT,RPAR,IPAR)
C
      IF(ERR .GT. 1.0D0)GO TO 600
C
C
C
C
C
C-----------------------------------------------------------------------
C     BLOCK 5
C     The step is successful.  No further action required.
C-----------------------------------------------------------------------
C
      RETURN
C
C
C
C
C
C-----------------------------------------------------------------------
C     BLOCK 6
C     The step is unsuccessful. Restore X, Y and YPRIME
C     to their original values.
C     Reduce the stepsize and try again, or exit with
C     an error flag if there have been many failures.
C-----------------------------------------------------------------------
C
600   CONTINUE
C
C     Restore X, Y and YPRIME.
C
      X = XOLD
      DO 610 I=1,NEQ
         Y(I)=PHI(I,1)
610      YPRIME(I)=PHI(I,2)
C
C     Test whether failure is due to nonlinear solver
C     or error test.
C
      IF(IERNLS .EQ. 0)GO TO 660
      IWM(LCFN)=IWM(LCFN)+1
C
C     The nonlinear solver failed to converge.
C     Determine the cause of the failure and take appropriate action.
C     If IERNLS .LT. 0, then return.  Otherwise, reduce the stepsize
C     and try again, unless too many failures have occurred.
C
      IF (IERNLS .LT. 0) GOTO 675
      NCF = NCF + 1
      R = 0.25D0
      H = H*R
      IF (NCF .LT. 10 .AND. DABS(H) .GE. HMIN) GO TO 690
      GO TO 675
C
C     The nonlinear solver converged, and the cause
C     of the failure was the error estimate
C     exceeding the tolerance.
C
660   NEF=NEF+1
      IWM(LETF)=IWM(LETF)+1
C
C     On first and subsequent error test failures,
C     Compute new stepsize based on differences
C     of the solution.
C
      R=0.90D0/(2.0D0*ERR+0.0001D0)
      R=MAX(0.1D0,MIN(0.5D0,R))
      H=H*R
      IF (ABS(H).GE.HMIN.AND.NEF.LT.10) GO TO 690
      GO TO 675
C
C
C     For all crashes, return.
C
C     Before returning, verify that the user has not set
C     IDID to a nonnegative value.  If the user has set IDID
C     to a nonnegative value, then reset IDID to be -12, indicating
C     a failure to compute the initial YPRIME.
C
675   CONTINUE
      IF (IDID .GE. 0) IDID = -12
      RETURN
C
C     Go back and try this step again.
C
690   GO TO 200
C
C------END OF SUBROUTINE DDINI-----------------------------------------
      END
