C Work performed under the auspices of the U.S. Department of Energy
C by Lawrence Livermore National Laboratory under contract number 
C W-7405-Eng-48.
C
C Copyright 1990 the Regents of the University of California.
C All rights reserved.
C
      SUBROUTINE DNEDID(X,Y,YPRIME,NEQ,
     *   RES,JACD,PDUM,H,WT,JSTART,IDID,RPAR,IPAR,PHIDUM,GADUM,DUMSVR,
     *   DELTA,E,WM,IWM,CJ,CJDUM,CJLDUM,S,
     *   UROUND,DUME,DUMS,DUMR,EPCON,JCALC,IDUMJ,KPDUM,
     *   NONNEG,NTEMP,NTYPE,IERNLS)
C
C***BEGIN PROLOGUE  DNEDID
C***REFER TO  DDASPK
C***DATE WRITTEN   891219   (YYMMDD)
C***REVISION DATE  900926   (YYMMDD)
C
C
C-----------------------------------------------------------------------
C***DESCRIPTION
C
C
C     DNEDID solves a nonlinear system of
C     algebraic equations of the form
C     G(X,Y,YPRIME) = 0 for the unknown Y and YPRIME.
C     An initial value for Y and initial guess for YPRIME are input.
C
C     The method used is a modified damped Newton scheme.
C
C     The parameters represent
C
C     X         -- Independent variable.
C     Y         -- Solution vector.
C     YPRIME    -- Derivative of solution vector.
C     NEQ       -- Number of unknowns.
C     RES       -- External user-supplied subroutine
C                  to evaluate the residual.  See RES description
C                  in DDASPK prologue.
C     JACD      -- External user-supplied routine to evaluate the
C                  Jacobian.  See JAC description for the case
C                  INFO(12) = 0 in the DDASPK prologue.
C     PDUM      -- Dummy argument.
C     H         -- Appropriate step size for next step.
C     WT        -- Vector of weights for error criterion.
C     JSTART    -- Indicates first call to this routine.
C                  If JSTART = 0, then this is the first call,
C                  otherwise it is not.
C     IDID      -- Completion flag, output by DNEDID.
C                  See IDID description in DDASPK prologue.
C     RPAR,IPAR -- Real and integer parameter arrays that
C                  are used for communication between the
C                  calling program and external user routines.
C                  They are not altered by DNEDID
C     PHIDUM    -- Dummy argument.
C     GADUM     -- Dummy argument.
C     DUMSVR    -- Dummy argument.
C     DELTA     -- Work vector for NLS of length NEQ.
C     E         -- Error accumulation vector for NLS of length NEQ.
C     WM,IWM    -- Real and integer arrays storing
C                  matrix information such as the matrix
C                  of partial derivatives, permutation
C                  vector, and various other information.
C     CJ        -- Parameter always proportional to 1/H.
C     CJDUM     -- Dummy argument.
C     CJLDUM    -- Dummy argument.
C     S         -- A scalar determined by the approximate rate
C                  of convergence of the Newton iteration and used
C                  in the convergence test for the Newton iteration.
C
C                  If RATE is defined to be an estimate of the
C                  rate of convergence of the Newton iteration,
C                  then S = RATE/(1.D0-RATE).
C
C                  The closer RATE is to 0., the faster the Newton
C                  iteration is converging; the closer RATE is to 1.,
C                  the slower the Newton iteration is converging.
C
C                  On the first Newton iteration with an up-dated
C                  preconditioner S = 100.D0, Thus the initial
C                  RATE of convergence is approximately 1.
C
C                  S is preserved from call to call so that the rate
C                  estimate from a previous step can be applied to
C                  the current step.
C     UROUND    -- Unit roundoff.
C     DUME      -- Dummy argument.
C     DUMS      -- Dummy argument.
C     DUMR      -- Dummy argument.
C     EPCON     -- Tolerance to test for convergence of the Newton
C                  iteration.
C     JCALC     -- Flag used to determine when to update
C                  the Jacobian matrix.  In general:
C
C                  JCALC = -1 ==> Call the DITMD routine to update
C                                 the Jacobian matrix.
C                  JCALC =  0 ==> Jacobian matrix is up-to-date.
C                  JCALC =  1 ==> Jacobian matrix is out-dated,
C                                 but DITMD will not be called unless
C                                 JCALC is set to -1.
C     IDUMJ     -- Dummy argument.
C     KP1       -- The current order(K) + 1;  updated across calls.
C     NONNEG    -- Flag to determine nonnegativity constraints.
C     NTEMP     -- Pointer to additional temporary storage space.
C     NTYPE     -- Identification code for the NLS routine.
C                   0  ==> modified Newton; direct solver.
C     IERNLS    -- Error flag for nonlinear solver.
C                   0  ==> nonlinear solver converged.
C                   1  ==> recoverable error inside nonlinear solver.
C                  -1  ==> unrecoverable error inside nonlinear solver.
C
C     All variables with "DUM" in their names are dummy variables
C     which are not used in this routine.
C
C     Following is a list and description of local variables which
C     may not have an obvious usage.  They are listed in roughly the
C     order they occur in this subroutine.
C
C     The following group of variables are passed as arguments to
C     the Newton iteration solver.  They are explained in greater detail
C     in DMNED:
C
C     TOLNEW, MULDEL, MAXIT, IERM, IERNEW
C
C     IERTYP -- Flag which tells whether this subroutine is correct.
C               0 ==> correct subroutine.
C               1 ==> incorrect subroutine.
C
C-----------------------------------------------------------------------
C
C***ROUTINES CALLED
C   DDWNRM, RES, DITMD, DMNED
C
C***END PROLOGUE  DNEDID
C
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION Y(*),YPRIME(*),WT(*)
      DIMENSION DELTA(*),E(*)
      DIMENSION WM(*),IWM(*)
      EXTERNAL  RES, JACD
C
      PARAMETER (LNRE=12)
      PARAMETER (LNJE=13)
      PARAMETER (LNNI=19)
C
      DATA MAXIT/10/,MJAC/5/
      DATA MULDEL/1/, DAMP/0.75D0/
C
C     Verify that this is the correct subroutine.
C
      IERTYP = 0
      IF (NTYPE .NE. 0) THEN
         IERTYP = 1
         GOTO 380
         ENDIF
C
C     Perform initializations.
C
      JCALC = -1
      IERNLS = 0
      M0=IWM(LNNI)
C
C     Predict the solution and derivative and compute the tolerance
C     for the Newton iteration.
C
      DO 200 I=1,NEQ
200     Y(I)=Y(I)+H*YPRIME(I)
      YNORM = DDWNRM (NEQ,Y,WT,RPAR,IPAR)
      TOLNEW = 100.D0*UROUND*YNORM
C
C     Call RES to initialize DELTA.
C
      IWM(LNRE) = IWM(LNRE) + 1
      CALL RES(X,Y,YPRIME,CJ,DELTA,IRES,RPAR,IPAR)
      IF (IRES .LT. 0) GO TO 380
C
C-----------------------------------------------------------------------
C     Entry point for updating the Jacobian with current
C     stepsize.
C----------------------------------------------------------------------
300   CONTINUE
C
C     Initialize all error flags to zero.
C
      IERJ = 0
      IRES = 0
      IERM = 0
      IERNEW = 0
C
C     If indicated, reevaluate the
C     iteration matrix PD = dG/dY + CJ*dG/dYPRIME
C     (where G(X,Y,YPRIME)=0). Set
C     JCALC to 0 as an indicator that
C     this has been done.
C
      IF (JCALC.EQ.-1) THEN
         IWM(LNJE)=IWM(LNJE)+1
         JCALC=0
         CALL DITMD(NEQ,X,Y,YPRIME,DELTA,CJ,H,
     *   IERJ,WT,E,WM,IWM,RES,IRES,UROUND,JACD,RPAR,
     *   IPAR,NTEMP)
         S=1000000.D0
         IF (IRES.LT.0) GO TO 380
         IF (IERJ.NE.0) GO TO 380
      ENDIF

C
C     Call the nonlinear newton solver with MJAC 
C     maximum iterations.
C
      MXNEWT = MIN(MAXIT-(IWM(LNNI)-M0),MJAC)
      CALL DMNED(X,Y,YPRIME,NEQ,
     *   RES,PDUM,WT,RPAR,IPAR,DUMSVR,
     *   DELTA,E,WM,IWM,CJ,
     *   DUMS,DUMR,DUME,EPCON,
     *   S,DAMP,TOLNEW,MULDEL,MXNEWT,
     *   IRES,IDUM,IERM,IERNEW)

      IF (IERM .GT. 0) THEN
C
C        MJAC iterations have been performed.
C        This is not necessarily an error condition.
C        The Jacobian must be updated every MJAC iterations,
C        unless MAXIT iterations have been performed, which
C        is an error condition.
C
         IF ( IWM(LNNI)-M0 .LT. MAXIT) THEN
C
C           Set JCALC to indicate that a new Jacobian is needed.
C           Call the RES routine to update DELTA.
C           Go back and do another iteration.
C
            JCALC = -1
            IWM(LNRE)=IWM(LNRE)+1
            CALL RES(X,Y,YPRIME,CJ,DELTA,IRES,RPAR,IPAR)
            IF (IRES .LT. 0) GO TO 380
            GO TO 300
         ENDIF
      ENDIF
C
      IF (IERNEW .NE. 0) GOTO 380
C
C     The iteration has converged.  If nonnegativity of solution is
C     required, set the solution nonnegative, if the perturbation
C     to do it is small enough.  If the change is too large, then
C     consider the corrector iteration to have failed.
C
370   IF(NONNEG .EQ. 0) GO TO 390
      DO 377 I = 1,NEQ
377      DELTA(I) = DMIN1(Y(I),0.0D0)
      DELNRM = DDWNRM(NEQ,DELTA,WT,RPAR,IPAR)
      IF(DELNRM .GT. EPCON) GO TO 380
      DO 378 I = 1,NEQ
         Y(I)=Y(I)-DELTA(I)
378      YPRIME(I)=YPRIME(I)-CJ*DELTA(I)
      GO TO 390
C
C
C     Unsuccessful exit from nonlinear solver.
C     No convergence with current iteration
C     matrix, or singular iteration matrix.
C     Compute IERNLS and IDID accordingly.
C
380   CONTINUE
      IF (IRES .LE. -2 .OR. IERTYP .NE. 0) THEN
         IERNLS = -1
         IF (IRES .LE. -2) IDID = -12
         IF (IERTYP .NE. 0) IDID = -15
      ELSE
         IERNLS = 1
         IF (IRES .LT. 0) IDID = -12
         IF (IERJ .NE. 0) IDID = -12
      ENDIF
C
390   CONTINUE
      RETURN
C
C------END OF SUBROUTINE DNEDID-----------------------------------------
      END
