

C Work performed under the auspices of the U.S. Department of Energy
C by Lawrence Livermore National Laboratory under contract number 
C W-7405-Eng-48.
C
C Copyright 1990 the Regents of the University of California.
C All rights reserved.
C
C This file contains a general purpose preconditioner for use in the
C solution of DAE systems by DDASPK.  The preconditioner is an
C approximation to a banded portion of the Jacobian matrix.
C It is intended as an example preconditioner, and likely will not
C be optimal for a wide class of problems.  
C
C This banded matrix is updated periodically and factored via the
C routine DJACK below.  The factored preconditioner is then used
C inside the DPSOL routine (listed after DJACK) to solve
C linear systems.
C
C The approximation is accomplished by employing the techniques normally
C used by DDASSL in generating a finite-difference banded Jacobian
C matrix, with the exception that the bandwidth parameters, ML and MU,
C used in this routine can be less than the actual bandwidth parameters
C of the Jacobian.
C
C When ML and MU are less than the actual bandwidth
C parameters, the computed matrix is only an approximation to the
C corresponding banded portion of the Jacobian matrix.
C This approximation will likely be accurate enough if the desired
C banded portion of the Jacobian is the dominate part of the
C entire matrix.
C
      SUBROUTINE DJACK (RES, IRES, NEQ, T, Y, YPRIME, REWT, SAVR,
     *  WK, H, CJ, WP, IWP, IER, RPAR, IPAR)
C
C***BEGIN PROLOGUE  DJACK
C***REFER TO  DDASPK
C***DATE WRITTEN   891204   (YYMMDD)
C***REVISION DATE  900122   (YYMMDD)
C***AUTHORS  Petzold, Linda R., Brown, Peter N.,  Hindmarsh, Alan C.,
C            Ulrich, Clement W.,
C            Computing and Mathematics Research Division
C            Lawrence Livermore National Laboratory
C            L-316, P.O. Box 808,
C            Livermore, CA.    94550
C
C***DESCRIPTION
C
C     This routine computes an approximation to a banded portion
C     of the iteration matrix PD=dG/dY+CJ*dG/dYPRIME
C     (where G(T,Y,YPRIME)=0).  Here PD is computed by numerical
C     finite differencing.
C
C     The diagonal, the first ML subdiagonals, and the first MU
C     super diagonals are computed.  If ML = MU = NEQ-1, then the
C     entire Jacobian will be computed. If ML = MU = 0, then only
C     the diagonal of the Jacobian will be approximated.
C
C     ML and MU are defined in the main driver and stored, respectively,
C     inside IPAR locations IPAR(LML) and IPAR(LMU).  See PARAMETER
C     declarations of of LML and LMU below.
C     
C     This banded portion of the iteration matrix is intended to serve
C     as a preconditioner for the iterative linear system solver.
C
C     The parameters have the following meanings.
C
C     RES      = External user-supplied subroutine
C                to evaluate the residual.  See RES description
C                in DDASPK prologue.
C     IRES     = Flag which is equal to zero if no illegal values
C                in RES, and less than zero otherwise.  See RES
C                description in DDASPK prologue.
C     NEQ      = Number of unknowns.
C     T        = Independent variable.
C     Y        = Array containing current value of Y.
C     YPRIME   = Array containing current derivative of Y.
C     REWT     = Vector of error weights for computing norms.
C     SAVR     = Current residual evaluated at (T,Y,YPRIME).
C     WK       = Work space (temporary) of length NEQ.
C     H        = Current step size.
C     CJ       = Scalar proportional to 1/H.
C     WP       = Real work space for matrices.  On output
C                it contains the LU decomposition
C                of the approximation to the iteration matrix.
C     IWP      = Integer work space containing
C                matrix information.
C     IER      = Variable which is .NE. 0 if iteration matrix is
C                singular, and 0 otherwise.
C     RPAR,IPAR= Real and integer parameter arrays that
C                are used for communication between the
C                calling program and external user routines (like
C                this one).
C
C***ROUTINES CALLED
C   D1MACH, DGBFA, RES
C
C***END PROLOGUE  DPSOL
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      EXTERNAL D1MACH,DGBFA,RES
      DIMENSION Y(*),YPRIME(*),SAVR(*),REWT(*),WK(*)
      DIMENSION WP(*),IWP(*),RPAR(*),IPAR(*)
C
C Pointers into IPAR:
C
      PARAMETER (LML=3, LMU=4, LNRE = 5)
C
C Compute UROUND, the unit roundoff error of the machine being used.
C
      UROUND = D1MACH(4)
C
C Compute LIPVT, an offset for banded storage integer work space.
C LIPVT points to the location where the pivot vector will
C start.
C
      LIPVT = 1
C
C Compute NPD and NTEMP, offsets for banded storage real work space.
C NPD points to the location where the banded matrix will start.
C NTEMP points to the location just past the minimum required
C banded storage space, LENPD, where extra work space is required.
C
      NPD = 1
      LENPD=(2*IPAR(LML)+IPAR(LMU)+1)*NEQ
      NTEMP=NPD + LENPD
C
C Initializations:
C
      IER = 0
      NPDM1=NPD-1
C
C Generate the required portion of the iteration matrix using
C finite difference methods.
C
      MBAND=IPAR(LML)+IPAR(LMU)+1
      MBA=MIN0(MBAND,NEQ)
      MEBAND=MBAND+IPAR(LML)
      MEB1=MEBAND-1
      MSAVE=(NEQ/MBAND)+1
      ISAVE=NTEMP-1
      IPSAVE=ISAVE+MSAVE
      IRES=0
      SQUR=DSQRT(UROUND)
      DO 540 J=1,MBA
         DO 510 N=J,NEQ,MBAND
          K= (N-J)/MBAND + 1
          WP(ISAVE+K)=Y(N)
          WP(IPSAVE+K)=YPRIME(N)
          DEL=SQUR*DMAX1(DABS(Y(N)),DABS(H*YPRIME(N)),
     *      DABS(1.D0/REWT(N)))
          DEL=DSIGN(DEL,H*YPRIME(N))
          DEL=(Y(N)+DEL)-Y(N)
          Y(N)=Y(N)+DEL
510       YPRIME(N)=YPRIME(N)+CJ*DEL
      CALL RES(T,Y,YPRIME,CJ,WK,IRES,RPAR,IPAR)
      IPAR(LNRE) = IPAR(LNRE) + 1
      IF (IRES .LT. 0) RETURN
      DO 530 N=J,NEQ,MBAND
          K= (N-J)/MBAND + 1
          Y(N)=WP(ISAVE+K)
          YPRIME(N)=WP(IPSAVE+K)
          DEL=SQUR*DMAX1(DABS(Y(N)),DABS(H*YPRIME(N)),
     *      DABS(1.D0/REWT(N)))
          DEL=DSIGN(DEL,H*YPRIME(N))
          DEL=(Y(N)+DEL)-Y(N)
          DELINV=1.0D0/DEL
          I1=MAX0(1,(N-IPAR(LMU)))
          I2=MIN0(NEQ,(N+IPAR(LML)))
          II=N*MEB1-IPAR(LML)+NPDM1
          DO 520 I=I1,I2
520         WP(II+I)=(WK(I)-SAVR(I))*DELINV
530      CONTINUE
540   CONTINUE
C
C Do LU decomposition of banded PD.
C
      CALL DGBFA(WP(NPD),MEBAND,NEQ,
     *    IPAR(LML),IPAR(LMU),IWP(LIPVT),IER)
      RETURN
C
C-----END OF SUBROUTINE DJACK-------------------------------------------
      END
C
C
C
C
      SUBROUTINE DPSOL (NEQ, T, Y, YPRIME, SAVR, WK, CJ, WGHT,
     *   WP, IWP, B, EPLIN, IER, RPAR, IPAR)
C
C***BEGIN PROLOGUE  DPSOL
C***REFER TO  DDASPK
C***DATE WRITTEN   891204   (YYMMDD)
C***REVISION DATE  900110   (YYMMDD)
C***AUTHORS  Petzold, Linda R., Brown, Peter N.,  Hindmarsh, Alan H.,
C            Ulrich, Clement W.,
C            Computing and Mathematics Research Division
C            Lawrence Livermore National Laboratory
C            L-316, P.O. Box 808,
C            Livermore, CA.    94550
C
C***DESCRIPTION
C
C     This routine interfaces to the linpack routine DGBSL.
C
C     The parameters have the following meanings.
C     NEQ      = Number of equations.
C     T        = Independent variable (not used).
C     Y        = Array containing current value of Y (not used).
C     YPRIME   = Array containing current derivative (not used).
C     SAVR     = Current residual evaluated at (T,Y,YPRIME) (not used).
C     WK       = Work space (temporary) of length NEQ (not used).
C     CJ       = Scalar proportional to 1/H (H = step size) (not used).
C     WGHT     = Vector of error weights for computing norms (not used).
C     WP       = Contains the LU decomposition
C                of the approximation to the iteration matrix.
C     IWP      = Integer work space containing
C                matrix information.
C     B        = right hand side on input; solution on output.
C     EPLIN    = tolerance on linear system solver (unused).
C     IER      = error flag.  Initialized to zero.
C     RPAR,IPAR= Real and integer parameter arrays that
C                are used for communication between the
C                calling program and external user routines (like
C                this one).
C
C***ROUTINES CALLED
C   DGBSL
C
C***END PROLOGUE  DJACK
C
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION B(*),WP(*),IWP(*),RPAR(*),IPAR(*)
      PARAMETER (LML=3, LMU=4)
C
C     Call banded matrix solver.
C
      MEBAND=2*IPAR(LML)+IPAR(LMU)+1
      CALL DGBSL(WP,MEBAND,NEQ,IPAR(LML),
     *  IPAR(LMU),IWP,B,0)
      RETURN
C
C------END OF SUBROUTINE DPSOL------------------------------------------
      END

