      double precision function d1mach(i)                               d1mach 4
c***begin prologue  d1mach                                              d1mach 5
c***date written   750101   (yymmdd)                                    d1mach 6
c***revision date  880315   (yymmdd)                                    bb8315 2
c***category no.  r1                                                    bb814745
c***keywords  library=slatec,type=double precision(r1mach-s d1mach-d),  bb814746
c             machine constants                                         bb814747
c***author  fox, p. a., (bell labs)                                     d1mach10
c           hall, a. d., (bell labs)                                    d1mach11
c           schryer, n. l., (bell labs)                                 d1mach12
c***purpose  returns double precision machine dependent constants       d1mach13
c***description                                                         d1mach14
c                                                                       d1mach15
c   d1mach can be used to obtain machine-dependent parameters           bb814748
c   for the local machine environment.  it is a function                bb814749
c   subprogram with one (input) argument, and can be called             bb814750
c   as follows, for example                                             bb814751
c                                                                       bb814752
c        d = d1mach(i)                                                  bb814753
c                                                                       bb814754
c   where i=1,...,5.  the (output) value of d above is                  bb814755
c   determined by the (input) value of i.  the results for              bb814756
c   various values of i are discussed below.                            bb814757
c                                                                       bb814758
c   d1mach( 1) = b**(emin-1), the smallest positive magnitude.          bb814759
c   d1mach( 2) = b**emax*(1 - b**(-t)), the largest magnitude.          bb814760
c   d1mach( 3) = b**(-t), the smallest relative spacing.                bb814761
c   d1mach( 4) = b**(1-t), the largest relative spacing.                bb814762
c   d1mach( 5) = log10(b)                                               bb814763
c                                                                       bb814764
c   assume double precision numbers are represented in the t-digit,     bb814765
c   base-b form                                                         bb814766
c                                                                       bb814767
c              sign (b**e)*( (x(1)/b) + ... + (x(t)/b**t) )             bb814768
c                                                                       bb814769
c   where 0 .le. x(i) .lt. b for i=1,...,t, 0 .lt. x(1), and            bb814770
c   emin .le. e .le. emax.                                              bb814771
c                                                                       bb814772
c   the values of b, t, emin and emax are provided in i1mach as         bb814773
c   follows:                                                            bb814774
c   i1mach(10) = b, the base.                                           bb814775
c   i1mach(14) = t, the number of base-b digits.                        bb814776
c   i1mach(15) = emin, the smallest exponent e.                         bb814777
c   i1mach(16) = emax, the largest exponent e.                          bb814778
c                                                                       bb814779
c   to alter this function for a particular environment,                bb814780
c   the desired set of data statements should be activated by           bb814781
c   removing the c from column 1.  also, the values of                  bb814782
c   d1mach(1) - d1mach(4) should be checked for consistency             bb814783
c   with the local operating system.                                    bb814784
c                                                                       bb814785
c***references  fox p.a., hall a.d., schryer n.l.,*framework for a      d1mach33
c                 portable library*, acm transactions on mathematical   d1mach34
c                 software, vol. 4, no. 2, june 1978, pp. 177-188.      d1mach35
c***routines called  xerror                                             d1mach36
c***end prologue  d1mach                                                d1mach37
c                                                                       d1mach38
      integer small(4)                                                  d1mach39
      integer large(4)                                                  d1mach40
      integer right(4)                                                  d1mach41
      integer diver(4)                                                  d1mach42
      integer log10(4)                                                  d1mach43
c                                                                       d1mach44
      double precision dmach(5)                                         d1mach45
c                                                                       d1mach46
      equivalence (dmach(1),small(1))                                   d1mach47
      equivalence (dmach(2),large(1))                                   d1mach48
      equivalence (dmach(3),right(1))                                   d1mach49
      equivalence (dmach(4),diver(1))                                   d1mach50
      equivalence (dmach(5),log10(1))                                   d1mach51
c                                                                       d1mach52
c     machine constants for the apollo                                  bb814786
c                                                                       bb814787
c     data small(1), small(2) / 16#00100000, 16#00000000 /              bb814788
c     data large(1), large(2) / 16#7fffffff, 16#ffffffff /              bb814789
c     data right(1), right(2) / 16#3ca00000, 16#00000000 /              bb814790
c     data diver(1), diver(2) / 16#3cb00000, 16#00000000 /              bb814791
c     data log10(1), log10(2) / 16#3fd34413, 16#509f79ff /              bb814792
c                                                                       bb814793
c     machine constants for the burroughs 1700 system                   bb814794
c                                                                       d1mach54
c     data small(1) / zc00800000 /                                      d1mach55
c     data small(2) / z000000000 /                                      d1mach56
c     data large(1) / zdffffffff /                                      d1mach58
c     data large(2) / zfffffffff /                                      d1mach59
c     data right(1) / zcc5800000 /                                      d1mach61
c     data right(2) / z000000000 /                                      d1mach62
c     data diver(1) / zcc6800000 /                                      d1mach64
c     data diver(2) / z000000000 /                                      d1mach65
c     data log10(1) / zd00e730e7 /                                      d1mach67
c     data log10(2) / zc77800dc0 /                                      d1mach68
c                                                                       d1mach69
c     machine constants for the burroughs 5700 system                   bb814795
c                                                                       d1mach71
c     data small(1) / o1771000000000000 /                               d1mach72
c     data small(2) / o0000000000000000 /                               d1mach73
c     data large(1) / o0777777777777777 /                               d1mach75
c     data large(2) / o0007777777777777 /                               d1mach76
c     data right(1) / o1461000000000000 /                               d1mach78
c     data right(2) / o0000000000000000 /                               d1mach79
c     data diver(1) / o1451000000000000 /                               d1mach81
c     data diver(2) / o0000000000000000 /                               d1mach82
c     data log10(1) / o1157163034761674 /                               d1mach84
c     data log10(2) / o0006677466732724 /                               d1mach85
c                                                                       d1mach86
c     machine constants for the burroughs 6700/7700 systems             bb814796
c                                                                       d1mach88
c     data small(1) / o1771000000000000 /                               d1mach89
c     data small(2) / o7770000000000000 /                               d1mach90
c     data large(1) / o0777777777777777 /                               d1mach92
c     data large(2) / o7777777777777777 /                               d1mach93
c     data right(1) / o1461000000000000 /                               d1mach95
c     data right(2) / o0000000000000000 /                               d1mach96
c     data diver(1) / o1451000000000000 /                               d1mach98
c     data diver(2) / o0000000000000000 /                               d1mach99
c     data log10(1) / o1157163034761674 /                               d1mac101
c     data log10(2) / o0006677466732724 /                               d1mac102
c                                                                       d1mac103
c     machine constants for the cdc 170/180 series using nos/ve         bb814797
c                                                                       bb814798
c     data small(1) / z"3001800000000000" /                             bb814799
c     data small(2) / z"3001000000000000" /                             bb814800
c     data large(1) / z"4ffefffffffffffe" /                             bb814801
c     data large(2) / z"4ffe000000000000" /                             bb814802
c     data right(1) / z"3fd2800000000000" /                             bb814803
c     data right(2) / z"3fd2000000000000" /                             bb814804
c     data diver(1) / z"3fd3800000000000" /                             bb814805
c     data diver(2) / z"3fd3000000000000" /                             bb814806
c     data log10(1) / z"3fff9a209a84fbcf" /                             bb814807
c     data log10(2) / z"3ffff7988f8959ac" /                             bb814808
c                                                                       bb814809
c     machine constants for the cdc 6000/7000 series                    bb814810
c                                                                       bb814811
c     data small(1) / o"00564000000000000000" /                         bb8315 3
c     data small(2) / o"00000000000000000000" /                         bb8315 4
c     data large(1) / o"37757777777777777777" /                         bb8315 5
c     data large(2) / o"37157777777777777777" /                         bb8315 6
c     data right(1) / o"15624000000000000000" /                         bb8315 7
c     data right(2) / o"00000000000000000000" /                         bb8315 8
c     data diver(1) / o"15634000000000000000" /                         bb8315 9
c     data diver(2) / o"00000000000000000000" /                         bb831510
c     data log10(1) / o"17164642023241175717" /                         bb831511
c     data log10(2) / o"16367571421742254654" /                         bb831512
c                                                                       bb811814
c     machine constants for the cray-1                                  bb814823
c                                                                       bb814824
c     data small(1) / 201354000000000000000b /                          bb814825
c     data small(2) / 000000000000000000000b /                          bb814826
c     data large(1) / 577767777777777777777b /                          bb814827
c     data large(2) / 000007777777777777774b /                          bb814828
c     data right(1) / 376434000000000000000b /                          bb814829
c     data right(2) / 000000000000000000000b /                          bb814830
c     data diver(1) / 376444000000000000000b /                          bb814831
c     data diver(2) / 000000000000000000000b /                          bb814832
c     data log10(1) / 377774642023241175717b /                          bb814833
c     data log10(2) / 000007571421742254654b /                          bb814834
c                                                                       bb814835
c     machine constants for the data general eclipse s/200              bb814836
c                                                                       bb814837
c     note - it may be appropriate to include the following card -      bb814838
c     static dmach(5)                                                   bb814839
c                                                                       bb814840
c     data small /    20k, 3*0 /                                        bb814841
c     data large / 77777k, 3*177777k /                                  bb814842
c     data right / 31420k, 3*0 /                                        bb814843
c     data diver / 32020k, 3*0 /                                        bb814844
c     data log10 / 40423k, 42023k, 50237k, 74776k /                     bb814845
c                                                                       bb814846
c     machine constants for the elxsi 6400                              bb814847
c     (assuming real*8 is the default double precision)                 bb814848
c                                                                       bb814849
c     data small(1), small(2) / '00100000'x,'00000000'x /               bb814850
c     data large(1), large(2) / '7fefffff'x,'ffffffff'x /               bb814851
c     data right(1), right(2) / '3cb00000'x,'00000000'x /               bb814852
c     data diver(1), diver(2) / '3cc00000'x,'00000000'x /               bb814853
c     data log10(1), log10(2) / '3fd34413'x,'509f79ff'x /               bb814854
c                                                                       bb814855
c     machine constants for the harris 220                              bb814856
c                                                                       bb814857
c     data small(1), small(2) / '20000000, '00000201 /                  bb814858
c     data large(1), large(2) / '37777777, '37777577 /                  bb814859
c     data right(1), right(2) / '20000000, '00000333 /                  bb814860
c     data diver(1), diver(2) / '20000000, '00000334 /                  bb814861
c     data log10(1), log10(2) / '23210115, '10237777 /                  bb814862
c                                                                       bb814863
c     machine constants for the honeywell 600/6000 series               bb814864
c                                                                       bb814865
c     data small(1), small(2) / o402400000000, o000000000000 /          bb814866
c     data large(1), large(2) / o376777777777, o777777777777 /          bb814867
c     data right(1), right(2) / o604400000000, o000000000000 /          bb814868
c     data diver(1), diver(2) / o606400000000, o000000000000 /          bb814869
c     data log10(1), log10(2) / o776464202324, o117571775714 /          bb814870
c                                                                       bb814871
c     machine constants for the hp 2100                                 bb814872
c     three word double precision option with ftn4                      bb814873
c                                                                       bb814874
c     data small(1), small(2), small(3) / 40000b,       0,       1 /    bb814875
c     data large(1), large(2), large(3) / 77777b, 177777b, 177776b /    bb814876
c     data right(1), right(2), right(3) / 40000b,       0,    265b /    bb814877
c     data diver(1), diver(2), diver(3) / 40000b,       0,    276b /    bb814878
c     data log10(1), log10(2), log10(3) / 46420b,  46502b,  77777b /    bb814879
c                                                                       bb814880
c     machine constants for the hp 2100                                 bb814881
c     four word double precision option with ftn4                       bb814882
c                                                                       bb814883
c     data small(1), small(2) /  40000b,       0 /                      bb814884
c     data small(3), small(4) /       0,       1 /                      bb814885
c     data large(1), large(2) /  77777b, 177777b /                      bb814886
c     data large(3), large(4) / 177777b, 177776b /                      bb814887
c     data right(1), right(2) /  40000b,       0 /                      bb814888
c     data right(3), right(4) /       0,    225b /                      bb814889
c     data diver(1), diver(2) /  40000b,       0 /                      bb814890
c     data diver(3), diver(4) /       0,    227b /                      bb814891
c     data log10(1), log10(2) /  46420b,  46502b /                      bb814892
c     data log10(3), log10(4) /  76747b, 176377b /                      bb814893
c                                                                       bb814894
c     machine constants for the hp 9000                                 bb814895
c                                                                       bb814896
c     data small(1), small(2) / 00040000000b, 00000000000b /            bb814897
c     data large(1), large(2) / 17737777777b, 37777777777b /            bb814898
c     data right(1), right(2) / 07454000000b, 00000000000b /            bb814899
c     data diver(1), diver(2) / 07460000000b, 00000000000b /            bb814900
c     data log10(1), log10(2) / 07764642023b, 12047674777b /            bb814901
c                                                                       bb814902
c     machine constants for the ibm 360/370 series,                     bb814903
c     the xerox sigma 5/7/9, the sel systems 85/86, and                 bb814904
c     the perkin elmer (interdata) 7/32.                                bb814905
c                                                                       bb814906
c     data small(1), small(2) / z00100000, z00000000 /                  bb814907
c     data large(1), large(2) / z7fffffff, zffffffff /                  bb814908
c     data right(1), right(2) / z33100000, z00000000 /                  bb814909
c     data diver(1), diver(2) / z34100000, z00000000 /                  bb814910
c     data log10(1), log10(2) / z41134413, z509f79ff /                  bb814911
c                                                                       bb814912
c     machine constants for the ibm pc                                  bb814913
c     assumes that all arithmetic is done in double precision           bb814914
c     on 8088, i.e., not in 80 bit form for the 8087.                   bb814915
c                                                                       bb814916
c     data small(1)           / 2.23d-308              /                bb814917
c     data large(1)           / 1.79d+308              /                bb814918
c     data right(1)           / 1.11d-16               /                bb814919
c     data diver(1)           / 2.22d-16               /                bb814920
c     data log10(1)           / 0.301029995663981195d0 /                bb814921
c                                                                       bb814922
c     machine constants for the pdp-10 (ka processor)                   bb814923
c                                                                       bb814924
c     data small(1), small(2) / "033400000000, "000000000000 /          bb814925
c     data large(1), large(2) / "377777777777, "344777777777 /          bb814926
c     data right(1), right(2) / "113400000000, "000000000000 /          bb814927
c     data diver(1), diver(2) / "114400000000, "000000000000 /          bb814928
c     data log10(1), log10(2) / "177464202324, "144117571776 /          bb814929
c                                                                       bb814930
c     machine constants for the pdp-10 (ki processor)                   bb814931
c                                                                       bb814932
c     data small(1), small(2) / "000400000000, "000000000000 /          bb814933
c     data large(1), large(2) / "377777777777, "377777777777 /          bb814934
c     data right(1), right(2) / "103400000000, "000000000000 /          bb814935
c     data diver(1), diver(2) / "104400000000, "000000000000 /          bb814936
c     data log10(1), log10(2) / "177464202324, "476747767461 /          bb814937
c                                                                       bb814938
c     machine constants for pdp-11 fortran supporting                   bb814939
c     32-bit integers (expressed in integer and octal).                 bb814940
c                                                                       bb814941
c     data small(1), small(2) /    8388608,           0 /               bb814942
c     data large(1), large(2) / 2147483647,          -1 /               bb814943
c     data right(1), right(2) /  612368384,           0 /               bb814944
c     data diver(1), diver(2) /  620756992,           0 /               bb814945
c     data log10(1), log10(2) / 1067065498, -2063872008 /               bb814946
c                                                                       bb814947
c     data small(1), small(2) / o00040000000, o00000000000 /            bb814948
c     data large(1), large(2) / o17777777777, o37777777777 /            bb814949
c     data right(1), right(2) / o04440000000, o00000000000 /            bb814950
c     data diver(1), diver(2) / o04500000000, o00000000000 /            bb814951
c     data log10(1), log10(2) / o07746420232, o20476747770 /            bb814952
c                                                                       bb814953
c     machine constants for pdp-11 fortran supporting                   bb814954
c     16-bit integers (expressed in integer and octal).                 bb814955
c                                                                       bb814956
c     data small(1), small(2) /    128,      0 /                        bb814957
c     data small(3), small(4) /      0,      0 /                        bb814958
c     data large(1), large(2) /  32767,     -1 /                        bb814959
c     data large(3), large(4) /     -1,     -1 /                        bb814960
c     data right(1), right(2) /   9344,      0 /                        bb814961
c     data right(3), right(4) /      0,      0 /                        bb814962
c     data diver(1), diver(2) /   9472,      0 /                        bb814963
c     data diver(3), diver(4) /      0,      0 /                        bb814964
c     data log10(1), log10(2) /  16282,   8346 /                        bb814965
c     data log10(3), log10(4) / -31493, -12296 /                        bb814966
c                                                                       bb814967
c     data small(1), small(2) / o000200, o000000 /                      bb814968
c     data small(3), small(4) / o000000, o000000 /                      bb814969
c     data large(1), large(2) / o077777, o177777 /                      bb814970
c     data large(3), large(4) / o177777, o177777 /                      bb814971
c     data right(1), right(2) / o022200, o000000 /                      bb814972
c     data right(3), right(4) / o000000, o000000 /                      bb814973
c     data diver(1), diver(2) / o022400, o000000 /                      bb814974
c     data diver(3), diver(4) / o000000, o000000 /                      bb814975
c     data log10(1), log10(2) / o037632, o020232 /                      bb814976
c     data log10(3), log10(4) / o102373, o147770 /                      bb814977
c                                                                       bb814978
c     machine constants for the sun                                     bb814979
c                                                                       bb814980
      data small(1), small(2) / z'00100000', z'00000000' /              bb814981
      data large(1), large(2) / z'7fefffff', z'ffffffff' /              bb814982
      data right(1), right(2) / z'3ca00000', z'00000000' /              bb814983
      data diver(1), diver(2) / z'3cb00000', z'00000000' /              bb814984
      data log10(1), log10(2) / z'3fd34413', z'509f79ff' /              bb814985
c                                                                       bb814986
c     machine constants for the univac 1100 series ftn compiler         bb814987
c                                                                       bb814988
c     data small(1), small(2) / o000040000000, o000000000000 /          bb814989
c     data large(1), large(2) / o377777777777, o777777777777 /          bb814990
c     data right(1), right(2) / o170540000000, o000000000000 /          bb814991
c     data diver(1), diver(2) / o170640000000, o000000000000 /          bb814992
c     data log10(1), log10(2) / o177746420232, o411757177572 /          bb814993
c                                                                       bb814994
c     machine constants for vax 11/780                                  bb814995
c     (expressed in integer and hexadecimal)                            bb814996
c     the hex format below may not be suitable for unix sysyems         bb814997
c     the integer format should be ok for unix systems                  bb814998
c                                                                       bb814999
c     data small(1), small(2) /        128,           0 /               bb815000
c     data large(1), large(2) /     -32769,          -1 /               bb815001
c     data right(1), right(2) /       9344,           0 /               bb815002
c     data diver(1), diver(2) /       9472,           0 /               bb815003
c     data log10(1), log10(2) /  546979738,  -805796613 /               bb815004
c                                                                       bb815005
c     data small(1), small(2) / z00000080, z00000000 /                  bb815006
c     data large(1), large(2) / zffff7fff, zffffffff /                  bb815007
c     data right(1), right(2) / z00002480, z00000000 /                  bb815008
c     data diver(1), diver(2) / z00002500, z00000000 /                  bb815009
c     data log10(1), log10(2) / z209a3f9a, zcff884fb /                  bb815010
c                                                                       bb815011
c     machine constants for vax 11/780 (g-floating)                     bb815012
c     (expressed in integer and hexadecimal)                            bb815013
c     the hex format below may not be suitable for unix sysyems         bb815014
c     the integer format should be ok for unix systems                  bb815015
c                                                                       bb815016
c     data small(1), small(2) /         16,           0 /               bb815017
c     data large(1), large(2) /     -32769,          -1 /               bb815018
c     data right(1), right(2) /      15552,           0 /               bb815019
c     data diver(1), diver(2) /      15568,           0 /               bb815020
c     data log10(1), log10(2) /  1142112243, 2046775455 /               bb815021
c                                                                       bb815022
c     data small(1), small(2) / z00000010, z00000000 /                  bb815023
c     data large(1), large(2) / zffff7fff, zffffffff /                  bb815024
c     data right(1), right(2) / z00003cc0, z00000000 /                  bb815025
c     data diver(1), diver(2) / z00003cd0, z00000000 /                  bb815026
c     data log10(1), log10(2) / z44133ff3, z79ff509f /                  bb815027
c                                                                       bb815028
c                                                                       bb815029
c***first executable statement  d1mach                                  bb815030
      if (i .lt. 1  .or.  i .gt. 5)                                     bb815031
     1   call xerror ('d1mach -- i out of bounds', 25, 1, 2)            bb815032
c                                                                       bb815033
      d1mach = dmach(i)                                                 bb815034
      return                                                            bb815035
c                                                                       bb815036
      end                                                               bb815037
