c *************************************************************
c *   FUNCVIS
c *
c *   Function to define viscosity in nodal point (i,j,k) with
c *   coordinates (x,y,z), temperature T and second invariant secinv
c *   If IVISTYPE>0, viscosity is dependent on the z-coordinate
c *   only, and this function is called for k=1,nz only.
c *   If IVISTYPE<-1, viscosity is dependent on all three coordinates
c *   and may be used to define viscosity that is dependent on
c *   temperature, pressure, strain rate and composition.
c *   To include composition dependent viscosity, information on the
c *   composition should be retrieved from the common blocks describing
c *   it. 
c *
c *   For any modifications: it is strongly recommended that any 
c *   parameters describing the rheological law are passed through
c *   user defined commons, and that the variables in MC3D commons
c *   (such as those in a3.h) are NOT used.
c * 
c *   PvK 950811
c *************************************************************
      function funcvis(ivistype,i,j,k,x,y,z,T,secinv) 
      use module_precision
      use module_giparam

      integer ivistype,i,j,k
      real*8 x,y,z,T,secinv,pi
c     *** Some constants for Busse et al. rheology
      parameter(rr=2.9957323,qq=225d0/rr-1d0/4d0*rr,gg=15d0/rr-1d0/2d0) 
      parameter(pi=3.1415926)
      common /pesimplerheology/ avisc,bvisc,wvisc

c
      avisc=2.3025851d0
      avisc=6.907755279d0
      bvisc=0.0d0
      wvisc=0.0d0

      funcvis = 1d0
      if (ivistype.eq.-20) then
c        *** Temperature dependent viscosity for Busse et al. benchmark
c        *** Garrett Ito, 5/95\
          v1=qq/(T+gg)
          v0=qq/(1d0/2d0+gg)
          funcvis = exp(v1-v0)
          return
      endif

      if (ivistype.eq.-2) then
c        *** T,p dependent rheology

       a0_lm=40.0d0
       a0_tz=4.0d0
       a0_um=one
       a1_lm=zero
       a2_lm=zero
       a3_lm=zero
       a1_tz=zero
       a1_um=zero
       slant2=7.d0
       slant1=50.d0
       dep2=1.789d0
       dep1=1.9727d0
       nz_vis=65
        r=2.20415d0
        gtz=half*(one-dtanh((dep2-r)*slant2))
        gum=half*(one-dtanh((dep1-r)*slant1))

        flm=a0_lm+a1_lm*r+a2_lm*r**2+a3_lm*r**3
        ftz=a0_tz+a1_tz*r
        fum=a0_um+a1_um*r

        nz_funcvis=(flm+(ftz-flm)*gtz+(fum-ftz)*gum)

        r=float(k-1)/float(nz_vis-1)+1.20415d0
        gtz=half*(one-dtanh((dep2-r)*slant2))
        gum=half*(one-dtanh((dep1-r)*slant1))
 
        flm=a0_lm+a1_lm*r+a2_lm*r**2+a3_lm*r**3
        ftz=a0_tz+a1_tz*r
        fum=a0_um+a1_um*r
        if(k.le.(nz_vis-5))then
         vis_amp=(flm+(ftz-flm)*gtz+(fum-ftz)*gum)/nz_funcvis
         funcvis = vis_amp*1000.*exp(-avisc*T+ bvisc*(1-z))
        else
         funcvis = 1000.
        endif

c         funcvis = 1000.*exp(-avisc*T+ bvisc*(1-z))
c        funcvis = 1d0/exp(bvisc) + x
c        funcvis = 1d0 + 1d0/exp(bvisc) + sin(2*pi*x/wvisc)
c        write(6,'(''funcvis: '',5f8.3)') avisc,bvisc,T,z,funcvis
         return
      endif

      if (ivistype.eq.-3) then
c        *** x, dependent rheology
         funcvis = x*49.5d0+1d0
      endif

      if (ivistype.eq.-4) then
         if(x.lt.2.)then
         funcvis = 1.0
         else
         funcvis = 10.0
         endif 
      endif

      if (ivistype.eq.-5) then
         if(x.lt.1.)then
         funcvis = 1.0
         else
         funcvis = 10.0
         endif
      endif

      if (ivistype.eq.-6) then
        delnu=100.-0.1
        var1=4.d0/44.d0
        var2=40.d0/44.d0
        if(z.lt.(40d0/44d0))then
          funcvis=1.0
        elseif(x.lt.(2d0/44d0))then
          funcvis=0.1
        elseif(x.le.(6d0/44d0))then
          funcvis=0.1+delnu*(sin(pi*(x-var1)/var1)+1.d0)*.5d0
        elseif(x.gt.(42d0/44d0))then
          funcvis=0.1
        elseif(x.ge.(38d0/44d0))then
          funcvis=0.1+delnu*(-sin(pi*(x-var2)/var1)+1.d0)*.5d0
        else
          funcvis=100.d0
        endif
      endif

      return
      end
